@@ -1,4 +1,4 @@
-# $Id: Iterator.pm 2666 2007-10-02 22:04:36Z andy $
+# $Id: Iterator.pm 2679 2007-10-03 21:08:48Z andy $
 package Parallel::Iterator;
 
 use warnings;
@@ -11,13 +11,14 @@ use Config;
 
 require 5.008;
 
-our $VERSION = '0.3.0';
+our $VERSION = '0.4.0';
 use base qw( Exporter );
 our @EXPORT_OK = qw( iterate iterate_as_array iterate_as_hash );
 
 my %DEFAULTS = (
     workers => ( $Config{d_fork} ? 10 : 0 ),
     onerror => 'die',
+    nowarn  => 0,
 );
 
 =head1 NAME
@@ -26,7 +27,7 @@ Parallel::Iterator - Simple parallel execution
 
 =head1 VERSION
 
-This document describes Parallel::Iterator version 0.3.0
+This document describes Parallel::Iterator version 0.4.0
 
 =head1 SYNOPSIS
 
@@ -231,6 +232,12 @@ of C<$@> thrown.
         $worker,
         \@jobs
     );
+    
+=item C<nowarn>
+
+Normally C<iterate> will issue a warning on systems on which fork is not
+available and fall back to single process mode. This option supresses
+that warning.
 
 =back
 
@@ -278,18 +285,28 @@ sub iterate {
       unless 'CODE' eq ref $options{onerror};
 
     if ( $options{workers} > 0 && $DEFAULTS{workers} == 0 ) {
-        warn "Fork not available, falling back to single process mode\n";
+        # TODO: Add nowarn option.
+        warn "Fork not available, falling back to single process mode\n"
+          unless $options{nowarn};
         $options{workers} = 0;
     }
 
     if ( $options{workers} == 0 ) {
         # Non-forking version
         return sub {
-            if ( my @next = $iter->() ) {
-                return ( $next[0], $worker->( @next ) );
-            }
-            else {
-                return;
+            while ( 1 ) {
+                if ( my @next = $iter->() ) {
+                    my $result = eval { $worker->( @next ) };
+                    if ( my $err = $@ ) {
+                        $options{onerror}->( $next[0], $err );
+                    }
+                    else {
+                        return ( $next[0], $result );
+                    }
+                }
+                else {
+                    return;
+                }
             }
         };
     }
